<?php
/**
 * This is a class for uploading or editing categeories image
 * Options name: advanced_categories_widget_($category->term_id)
 * $term_id is the category id.
 * To access option please use:
 * get_option('advanced_categories_widget_' . $category->term_id );
 *
 * Version: 1.0
 * Author: zourbuth
 * Author URI: http://zourbuth.com
 * License: Under GPL2
 */

class Advanced_Categories_Image {

	/**
	 * @var boolean
	 **/
	protected $is_edit;

	/**
	 * Constructor for this class. 
	 **/
	function __construct()  {
		$this->is_edit = false;
		if ( is_admin() ) {
			
			// hacks the form element
			$this->add_action( 'load-categories.php', 'start_buffer' );
			$this->add_action( 'load-edit-tags.php', 'start_buffer' );
			$this->add_action( 'load-edit-tags.php', 'advanced_categories_widget_edit_tags_style' );
			
			// form fields
			$this->add_action( 'edit_category_form', 'edit_term_form' );
			$this->add_action( 'edit_tag_form', 'edit_term_form' );
			$this->add_action( 'add_tag_form', 'edit_term_form' );

			// action uploads, etc
			$this->add_action( 'edited_term', null, null, 2 );
			add_filter( 'gettext', array(&$this, 'replace_text_in_thickbox' ), 1, 3 );
		}
	}
	
	/**
	 * Register a WordPress action and map it back to the calling object
	 *
	 * @param string $action Name of the action
	 * @param string $function Function name (optional)
	 * @param int $priority WordPress priority (optional)
	 * @param int $accepted_args Number of arguments the function accepts (optional)
	 * @return void
	 **/
	function add_action ($action, $function = '', $priority = 10, $accepted_args = 1) {
		if ( $priority === null )
			$priority = 10;
		add_action ($action, array (&$this, $function == '' ? $action : $function), $priority, $accepted_args);
	}
	
	/* Push the widget stylesheet into widget admin page */
	function advanced_categories_widget_edit_tags_style() {
		wp_enqueue_style('thickbox');
		wp_enqueue_style( 'advanced-categories-widget-edit-tags', ADVANCED_CATEGORIES_WIDGET_URL . 'css/edit-tags.css' );
		wp_enqueue_script('media-upload');
		wp_enqueue_script('thickbox');
		wp_register_script('advanced-categories-uploader', ADVANCED_CATEGORIES_WIDGET_URL .'js/image-uploader.js', array('jquery','media-upload','thickbox'));
		wp_enqueue_script('advanced-categories-uploader');
	}

	/* This function will turn output buffering on */
	public function start_buffer() {
		$this->is_edit = true;
	
		if ( empty( $_GET['page'] ) ) 
			ob_start( array( & $this, 'modify_buffer' ) );
	}
	
	function replace_text_in_thickbox($translated_text, $source_text, $domain) {
		if ( $this->category_on_edit() ) {
			if ('Insert into Post' == $source_text) {
				return __('Add Image', $this->pluginDomain );
			}
		}
		return $translated_text;
	}
	
	function category_on_edit() {
		if ( isset($_SERVER['HTTP_REFERER']) && strpos($_SERVER['HTTP_REFERER'],'taxonomy_image') !== false ) {
			return true;
		} elseif ( isset($_REQUEST['_wp_http_referer']) && strpos($_REQUEST['_wp_http_referer'],'taxonomy_image') !== false ) {
			return true;
		} elseif ( isset($_REQUEST['widget_id']) && strpos($_REQUEST['widget_id'],'taxonomy_image') !== false ) {
			return true;
		}
		return false;
	}
	
	/* Output the buffering and replace specified strings */
	public function modify_buffer( $buffer ) {
		$buffer = str_replace( 
			array( '<form name="editcat"', '<form name="edittag"', '<table class="form-table">' ),
			array( '<form name="editcat" enctype="multipart/form-data"', '<form name="edittag" enctype="multipart/form-data"', 
			'<table class="form-table advance-category-left">' ), 
			$buffer );
		return $buffer;
	}

	public function edit_term_form( $term ) {
		$screen = get_current_screen();
		
		if ( is_object( $term ) && ! in_array( $screen->taxonomy, $this->get_selected_taxonomies() ) )
			return;
			
		$taxonomy = get_taxonomy( $screen->taxonomy );
		
		$action = @ $_GET[ 'action' ];
		if ( ! $action ) {
			echo '<p id="advanced_categories_widget">';
			printf( __( "Please created first this %s." , 'advanced-categories-widget' ), $taxonomy->labels->singular_name );
			echo '</p>';
			return;
		}
		
		/* get category info */
		$images_base_url = $this->term_images_base_url();
		
		/* check for upload max size for form data */
		$max_upload_size = $this->max_file_upload();
		
		/* get cat id */
		$term_id = $this->request_term_id();

		/* check for category image if available */
		$cat_image = get_option( 'advanced_categories_widget_' . $term_id );
		if ( !empty ( $cat_image ) )
			$term_image = get_option( 'advanced_categories_widget_' . $term_id );
			
		/* Load the admin page for uploading the categories image */
		require( ADVANCED_CATEGORIES_WIDGET_DIR . 'advanced-categories-widget-admin.php' );
	}
	
	public function edited_term( $term_id, $tt_id ) {
		/* If the nonce isn't present or incorrect, don'just return */
		if ( ! wp_verify_nonce( $_REQUEST[ '_acw_nonce' ], 'advanced_categories_widget') ) 
			return;

		// Deal with any file deletion *before* any upload
		$this->process_deletion();
		// Deal with any file upload
		$this->process_upload();
	}
	
	protected function process_deletion() {
		// Check that the delete button was actually pressed.
		if ( ! $this->was_button_pressed( 'advance-category-image-delete' ) ) 
			return;

		// Delete button was pressed, and we verified the Nonce earlier. Kindly proceed.
		
		$term_id = $this->request_term_id();
		//  image directory
		$base_dir = $this->images_base_dir();
		$term_image_original = trailingslashit( $base_dir ) . get_option('advanced_categories_widget_' . $term_id );
		
		unlink( $term_image_original );
		
		// delete options for database
		delete_option( 'advanced_categories_widget_' . $term_id );
	}
	
	protected function process_upload() {
		$term_id = $this->request_term_id();
		$file_img = $_POST[ 'taxonomy-image' ]; // Update categoryies image to database
		update_option( 'advanced_categories_widget_' . $term_id, $file_img );
	}

	protected function images_base_dir() {
		// Where should the dir be? Get the base WP uploads dir
		$wp_upload_dir = wp_upload_dir();
		$base_dir = $wp_upload_dir[ 'basedir' ];
		// Append our subdir
		$dir = $base_dir . '/advanced-categories-widget';
		// Does the dir exist? (If not, then make it)
		if ( ! file_exists( $dir ) ) {
			mkdir( $dir );
		}
		// Now return it
		return $dir;
	}

	protected function term_images_base_url() {
		// Where should the dir be? Get the base WP uploads dir
		$wp_upload_dir = wp_upload_dir();
		$base_url = $wp_upload_dir[ 'baseurl' ];
		
		// go to the plugin upload directory
		$url = $base_url . '/advanced-categories-widget';
		return $url;
	}
	
	protected function max_file_upload() {
		$upload_max_filesize = ini_get( 'upload_max_filesize' );
		$post_max_size = ini_get( 'post_max_size' );
		return min( $post_max_size, $upload_max_filesize );
	}

	protected function request_term_id() {
		$screen = get_current_screen();
		$term_id = @ $_REQUEST[ 'tag_ID' ];
		if ( ! $term_id ) 
			return false;
		return (int) $term_id;
	}
	
	/**
	 * Returns the filename for an image, used by the methods to
	 * get the filepath and URL.
	 *
	 * @param int $term_id The term ID of the image
	 * @param string $type Either 'original' or 'thumb' 
	 * @return void
	 * @author Simon Wheatley
	 **/
	protected function get_image_filename( $term_id, $type ) {
		if ( ! $data = $this->get_data( $term_id ) )
			return false;
		if ( ! isset( $data[ $type ] ) )
			return false;
		return $data[ $type ];
	}

	/**
	 * Returns the URL for an image file.
	 *
	 * @param int $term_id The ID of the term we want an image for 
	 * @param string $type The type of image
	 * @return string|bool A URL on success, false on failure
	 * @author Simon Wheatley
	 **/
	protected function get_image_url( $term_id, $type ) {
		if ( ! $filename = $this->get_image_filename( $term_id, $type ) )
			return false;
		$base_url = $this->term_images_base_url();
		
		$file_name = get_option('advanced_categories_widget_' . $term_id );
		return "$base_url/$file_name";

	}

	/**
	 * Returns the filepath for an image file.
	 *
	 * @param int $term_id The ID of the term we want an image for 
	 * @param string $type The type of image
	 * @return string|bool A filesystem path on success, false on failure
	 * @author Simon Wheatley
	 **/
	protected function get_image_filepath( $term_id, $type ) {
		if ( ! $filename = $this->get_image_filename( $term_id, $type ) )
			return false;
		$base_dir = $this->images_base_dir();
		return "$base_dir/$filename";
	}
	
	/**
	 * Saves a data file named as per the term ID.
	 *
	 * @param int $term_id The term ID to save the data file for 
	 * @param array $data The data to save
	 * @return void
	 **/
	protected function save_data( $term_id, $data ) {
		$base_dir = $this->images_base_dir();
		$file_path = "$base_dir/$term_id.dat";
		$serialized = serialize( $data );
		file_put_contents( $file_path, $serialized );
		wp_cache_set( $term_id, $data, 'acw' );
	}
	
	protected function get_selected_taxonomies() {
		$option = get_option( 'ciii_taxonomies', array( '1' => '1', 'taxonomies' => array( 'category' ) ) );
		return (array) $option[ 'taxonomies' ];
	}

	protected function was_button_pressed( $name, $post_only = false ) {
		if ( $post_only ) {
			$test_array = & $_POST;
		} else {
			$test_array = & $_REQUEST;
		}
		if ( @ $test_array[ $name ] != '' ) {
			return true;
		}
		if ( @ $test_array[$name . '_x'] != '' ) {
			return true;
		}
		return false;
	 }
}

/**
 * Create new instance
 **/
$Advanced_Categories_Image = new Advanced_Categories_Image();
?>