<?php
/**
 * The Categories widget replaces the default WordPress Categories widget. This version gives total
 * control over the output to the user by allowing the input of all the arguments typically seen
 * in the wp_list_categories() function.
 *
 */
class zFrame_Widget_Categories extends WP_Widget {

	/**
	 * Prefix for the widget.
	 * @since 0.7.0
	 */
	var $prefix;

	/**
	 * Textdomain for the widget.
	 * @since 0.7.0
	 */
	var $textdomain;

	/**
	 * Set up the widget's unique name, ID, class, description, and other options.
	 * @since 0.6.0
	 */
	function zFrame_Widget_Categories() {
	
		/* Give your own prefix name eq. your-theme-name- */
		$prefix = '';
		
		/* Set up the widget options. */
		$widget_options = array(
			'classname' => 'advanced_categories',
			'description' => esc_html__( '[+] Advanced categories widget that gives you total control over the output of your category links.', $this->textdomain )
		);

		/* Set up the widget control options. */
		$control_options = array(
			'width' => 460,
			'height' => 350,
			'id_base' => "{$this->prefix}advanced_categories"
		);

		/* Create the widget. */
		$this->WP_Widget( "{$this->prefix}advanced_categories", esc_attr__( 'Advanced Categories', $this->textdomain ), $widget_options, $control_options );
		
		/* load the widget stylesheet for the widgets admin screen. */
		add_action( 'load-widgets.php', array(&$this, 'advanced_categories_widget_admin_style') );
		
		/* print the user costum style sheet. */
		if ( is_active_widget(false, false, $this->id_base) ) {
			wp_enqueue_style( 'acw-style', ADVANCED_CATEGORIES_WIDGET_URL . 'css/widget.css' );
			add_action( 'wp_head', array( &$this, 'print_script') );
		}
	}

	/* Push the widget stylesheet widget.css into widget admin page */
	function advanced_categories_widget_admin_style() {		
		wp_enqueue_style( 'thickbox' );
		wp_print_scripts( 'jquery' );
		wp_enqueue_script( 'thickbox' );	
		wp_enqueue_style( 'custom-post-admin', ADVANCED_CATEGORIES_WIDGET_URL . 'css/widget-admin.css' );
		wp_enqueue_script( 'acw-script', ADVANCED_CATEGORIES_WIDGET_URL . 'js/jquery.widget.js' );
	}
	
	/* Creates the end HTML for widget */
	function advanced_categories_widget_html( $args = array() ) {
		$args = wp_parse_args( $args );
		$args['walker'] = new Walker_Advance_Category_Widget;
		$output = wp_list_categories( $args );
		if ( $output ) return $output;
	}
	
	function print_script() {
		$all_widgets = $this->get_settings();
		foreach ($all_widgets as $key => $setting){
			$widget_id = $this->id_base . '-' . $key;
			if( is_active_widget( false, $widget_id, $this->id_base ) ){
			
				// Print the widget background image
				echo '<style type="text/css">';
					if ( $setting['bgImage'] )		echo '#' . $this->id . ' {background: url(' . $setting['bgImage'] . ') no-repeat scroll 0 0 transparent}';
				echo '</style>';
				
				// Print the widget style adnd script
				if ( !empty( $setting['customstylescript'] ) ) echo $setting['customstylescript'];
			}
		}
	}
	
	/**
	 * Outputs the widget based on the arguments input through the widget controls.
	 * @since 0.6.0
	 */
	function widget( $args, $instance ) {
		extract( $args );

		/* Set up the arguments for wp_list_categories(). */
		$args = array(
			'show_title'			=> !empty( $instance['show_title'] ) ? true : false,
			'title_icon'			=> $instance['title_icon'],
			'style' 				=> $instance['style'],
			'orderby' 				=> $instance['orderby'],
			'order' 				=> $instance['order'],
			'include' 				=> !empty( $instance['include'] ) ? join( ', ', $instance['include'] ) : '',
			'exclude' 				=> !empty( $instance['exclude'] ) ? join( ', ', $instance['exclude'] ) : '',
			'exclude_tree' 			=> $instance['exclude_tree'],
			'depth' 				=> intval( $instance['depth'] ),
			'number' 				=> intval( $instance['number'] ),
			'child_of' 				=> intval( $instance['child_of'] ),
			'current_category' 		=> intval( $instance['current_category'] ),
			'feed' 					=> $instance['feed'],
			'feed_type' 			=> $instance['feed_type'],
			'feed_image' 			=> esc_url( $instance['feed_image'] ),
			'search'				=> $instance['search'],
			'hierarchical' 			=> !empty( $instance['hierarchical'] ) ? true : false,
			'use_desc_for_title'	=> !empty( $instance['use_desc_for_title'] ) ? true : false,
			'show_description'		=> !empty( $instance['show_description'] ) ? true : false,
			'show_last_update' 		=> !empty( $instance['show_last_update'] ) ? true : false,
			'show_count'			=> !empty( $instance['show_count'] ) ? true : false,
			'hide_empty'			=> !empty( $instance['hide_empty'] ) ? true : false,
			'title_li'				=> false,
			'icon_height' 			=> $instance['icon_height'],
			'icon_width' 			=> $instance['icon_width'],
			'bgImage'				=> $instance['bgImage'],
			'toggle_active'			=> $instance['toggle_active'],
			'intro_text' 			=> $instance['intro_text'],
			'outro_text' 			=> $instance['outro_text'],
			'customstylescript'		=> $instance['customstylescript'],
			'echo'					=> false
		);

		// Output the theme's widget wrapper
		echo $before_widget;

		// If a title was input by the user, display it.
		if ( !empty( $instance['title_icon'] ) )
			$titleIcon = '<img class="titleIcon" alt="" src="' . $instance['title_icon'] . '" />';
		else
			$titleIcon = '';	
		
		// If a title was input by the user, display it
		if ( !empty( $instance['title'] ) )
			echo $before_title . $titleIcon . apply_filters( 'widget_title',  $instance['title'], $instance, $this->id_base ) . $after_title;

		// Print intro text if exist
		if ( !empty( $instance['intro_text'] ) )
			echo '<p class="'. $this->id . '-intro-text">' . $instance['intro_text'] . '</p>';
			
		// Get the categories list
		$categories = str_replace( array( "\r", "\n", "\t" ), '', $this->advanced_categories_widget_html( $args ) );

		// If 'list' is the user-selected style, wrap the categories in an unordered list
		if ( 'list' == $args['style'] )
			$categories = '<ul class="xoxo categories advanced-categories-widget">' . $categories .  '</ul><!-- .xoxo .categories -->';

		// Output the categories list
		echo $categories;
		
		// Print outro text if exist
		if ( !empty( $instance['outro_text'] ) )
			echo '<p class="'. $this->id . '-outro_text">' . $instance['outro_text'] . '</p>';

		// Close the theme's widget wrapper
		echo $after_widget;
	}

	/**
	 * Updates the widget control options for the particular instance of the widget.
	 * @since 0.6.0
	 */
	function update( $new_instance, $old_instance ) {
		$instance = $old_instance;

		/* Set the instance to the new instance. */
		$instance = $new_instance;

		/* If new taxonomy is chosen, reset includes and excludes. */
		if ( $instance['taxonomy'] !== $old_instance['taxonomy'] && '' !== $old_instance['taxonomy'] ) {
			$instance['include'] 		= array();
			$instance['exclude'] 		= array();
		}

		$instance['title'] 				= strip_tags( $new_instance['title'] );
		$instance['show_title'] 		= ( isset( $new_instance['show_title'] ) ? 1 : 0 );
		$instance['title_icon']			= strip_tags( $new_instance['title_icon'] );
		$instance['exclude_tree'] 		= strip_tags( $new_instance['exclude_tree'] );
		$instance['depth'] 				= strip_tags( $new_instance['depth'] );
		$instance['number'] 			= strip_tags( $new_instance['number'] );
		$instance['child_of'] 			= strip_tags( $new_instance['child_of'] );
		$instance['current_category'] 	= strip_tags( $new_instance['current_category'] );
		$instance['feed'] 				= strip_tags( $new_instance['feed'] );
		$instance['feed_image'] 		= strip_tags( $new_instance['feed_image'] );
		$instance['search'] 			= strip_tags( $new_instance['search'] );

		$instance['hierarchical'] 		= ( isset( $new_instance['hierarchical'] ) ? 1 : 0 );
		$instance['use_desc_for_title'] = ( isset( $new_instance['use_desc_for_title'] ) ? 1 : 0 );
		$instance['show_description'] 	= ( isset( $new_instance['show_description'] ) ? 1 : 0 );
		$instance['show_last_update'] 	= ( isset( $new_instance['show_last_update'] ) ? 1 : 0 );
		$instance['show_count'] 		= ( isset( $new_instance['show_count'] ) ? 1 : 0 );
		$instance['hide_empty'] 		= ( isset( $new_instance['hide_empty'] ) ? 1 : 0 );
		$instance['icon_height'] 		= $new_instance['icon_height'];
		$instance['icon_width'] 		= $new_instance['icon_width'];
		$instance['bgImage'] 			= $new_instance['bgImage'];
		$instance['toggle_active'] 		= $new_instance['toggle_active'];
		$instance['intro_text'] 		= $new_instance['intro_text'];
		$instance['outro_text'] 		= $new_instance['outro_text'];
		$instance['customstylescript']	= $new_instance['customstylescript'];
		
		return $instance;
	}

	/**
	 * Displays the widget control options in the Widgets admin screen.
	 * @since 0.6.0
	 */
	function form( $instance ) {

		/* Set up the default form values. */
		$defaults = array(
			'title' 			=> esc_attr__( 'Categories', $this->textdomain ),
			'show_title' 		=> true,
			'title_icon'		=> '',
			'style' 			=> 'list',
			'include' 			=> array(),
			'exclude' 			=> array(),
			'exclude_tree'		 => '',
			'child_of' 			=> '',
			'current_category'	=> '',
			'search' 		=> '',
			'hierarchical' 	=> true,
			'hide_empty' 	=> true,
			'order' 		=> 'ASC',
			'orderby' 		=> 'name',
			'depth' 		=> 0,
			'number' 		=> '',
			'feed' 			=> '',
			'feed_type' 	=> '',
			'feed_image' 	=> '',
			'use_desc_for_title'	=> false,
			'show_description' 		=> true,
			'show_last_update' 		=> false,
			'show_count' 			=> false,
			'icon_height' 		=> '60',
			'icon_width' 		=> '60',
			'bgImage' 			=> '',
			'toggle_active'		=> array( 0 => true, 1 => false, 2 => false, 3 => false, 4 => false ),
			'intro_text' 		=> '',
			'outro_text' 		=> '',
			'customstylescript'	=> ''
		);

		/* Merge the user-selected arguments with the defaults. */
		$instance = wp_parse_args( (array) $instance, $defaults );

		/* <select> element options. */
		$taxonomies = get_taxonomies( array( 'show_tagcloud' => true ), 'objects' );
		$terms = get_terms( 'category' );
		$style = array( 'list' => esc_attr__( 'List', $this->textdomain ), 'none' => esc_attr__( 'None', $this->textdomain ) );
		$order = array( 'ASC' => esc_attr__( 'Ascending', $this->textdomain ), 'DESC' => esc_attr__( 'Descending', $this->textdomain ) );
		$orderby = array( 'count' => esc_attr__( 'Count', $this->textdomain ), 'ID' => esc_attr__( 'ID', $this->textdomain ), 'name' => esc_attr__( 'Name', $this->textdomain ), 'slug' => esc_attr__( 'Slug', $this->textdomain ), 'term_group' => esc_attr__( 'Term Group', $this->textdomain ) );
		$feed_type = array( '' => '', 'atom' => esc_attr__( 'Atom', $this->textdomain ), 'rdf' => esc_attr__( 'RDF', $this->textdomain ), 'rss' => esc_attr__( 'RSS', $this->textdomain ), 'rss2' => esc_attr__( 'RSS 2.0', $this->textdomain ) );
		$intro_text	= esc_textarea($instance['intro_text']);
		$outro_text	= esc_textarea($instance['outro_text']);
		?>
		
		<div class="pluginName">Advanced Categories Widget<span class="pluginVersion"><?php echo ADVANCED_CATEGORIES_WIDGET_VERSION; ?></span></div>
		
		<script type="text/javascript">
			// Tabs function
			jQuery(document).ready(function($){
				// Tabs function
				$('ul.nav-tabs li').each(function(i) {
					$(this).bind("click", function(){
						var liIndex = $(this).index();
						var content = $(this).parent("ul").next().children("li").eq(liIndex);
						$(this).addClass('active').siblings("li").removeClass('active');
						$(content).show().addClass('active').siblings().hide().removeClass('active');
	
						$(this).parent("ul").find("input").val(0);
						$('input', this).val(1);
					});
				});
				
				$('#<?php echo $this->id; ?>roleWrapper').sortable({ 
					items: '.role', 
					placeholder: 'placeholder'
				});
				
				// Farbtastic function
				$("#tupro-<?php echo $this->id; ?> .pickcolor").click(function() {
					$(this).next().slideToggle();					
					$(this).next().farbtastic($(this).prev());	
					return false;
				});
				$('html').click(function() { $('.farbtastic-wrapper').fadeOut(); });
				$('.farbtastic').click(function(event){ event.stopPropagation(); });
				
				// Image uploader/picker/remove
				$("#tupro-<?php echo $this->id; ?> a.addImage").totalAddImages();
				$("#tupro-<?php echo $this->id; ?> a.removeImage").totalRemoveImages();
				
				// Widget background
				$("#tupro-<?php echo $this->id; ?>").closest(".widget-inside").addClass("tuproWidgetBg");
			});
		</script>		

		<div id="tupro-<?php echo $this->id ; ?>" class="tuproControls tabbable tabs-left">
			
			<ul class="nav nav-tabs">
				<li class="<?php if ( $instance['toggle_active'][0] ) : ?>active<?php endif; ?>">General<input type="hidden" name="<?php echo $this->get_field_name( 'toggle_active' ); ?>[]" value="<?php echo esc_attr( $instance['toggle_active'][0] ); ?>" /></li>
				<li class="<?php if ( $instance['toggle_active'][1] ) : ?>active<?php endif; ?>">Advanced<input type="hidden" name="<?php echo $this->get_field_name( 'toggle_active' ); ?>[]" value="<?php echo esc_attr( $instance['toggle_active'][1] ); ?>" /></li>				
				<li class="<?php if ( $instance['toggle_active'][2] ) : ?>active<?php endif; ?>">Customs<input type="hidden" name="<?php echo $this->get_field_name( 'toggle_active' ); ?>[]" value="<?php echo esc_attr( $instance['toggle_active'][2] ); ?>" /></li>
				<li class="<?php if ( $instance['toggle_active'][3] ) : ?>active<?php endif; ?>">Supports<input type="hidden" name="<?php echo $this->get_field_name( 'toggle_active' ); ?>[]" value="<?php echo esc_attr( $instance['toggle_active'][3] ); ?>" /></li>
			</ul>			
			
			<ul class="tab-content">
				<li class="tab-pane <?php if ( $instance['toggle_active'][0] ) : ?>active<?php endif; ?>">
					<ul>
						<li>
							<label for="<?php echo $this->get_field_id( 'title' ); ?>"><?php _e( 'Title', $this->textdomain ); ?></label>
							<input type="text" class="widefat" id="<?php echo $this->get_field_id( 'title' ); ?>" name="<?php echo $this->get_field_name( 'title' ); ?>" value="<?php echo esc_attr( $instance['title'] ); ?>" />
						</li>
						<li>
							<label for="<?php echo $this->get_field_id( 'show_title' ); ?>">
							<input class="checkbox" type="checkbox" <?php checked( $instance['show_title'], true ); ?> id="<?php echo $this->get_field_id( 'show_title' ); ?>" name="<?php echo $this->get_field_name( 'show_title' ); ?>" /> <?php _e( 'Show title', $this->textdomain ); ?></code></label>
							<span class="controlDesc"><?php _e( 'Show or hide the category title.', $this->textdomain ); ?></span>
						</li>
						<li>
							<label for="<?php echo $this->get_field_id( 'show_description' ); ?>">
							<input class="checkbox" type="checkbox" <?php checked( $instance['show_description'], true ); ?> id="<?php echo $this->get_field_id( 'show_description' ); ?>" name="<?php echo $this->get_field_name( 'show_description' ); ?>" /> <?php _e( 'Show description', $this->textdomain ); ?></label>
							<span class="controlDesc"><?php _e( 'For displaying the categories description.', $this->textdomain ); ?></span>
						</li>
						<li>
							<label for="<?php echo $this->get_field_id( 'style' ); ?>"><?php _e( 'Style', $this->textdomain ); ?></label> 
							<span class="controlDesc"><?php _e( 'With HTML list or none.', $this->textdomain ); ?></span>
							<select class="smallfat" id="<?php echo $this->get_field_id( 'style' ); ?>" name="<?php echo $this->get_field_name( 'style' ); ?>">
								<?php foreach ( $style as $option_value => $option_label ) { ?>
									<option value="<?php echo esc_attr( $option_value ); ?>" <?php selected( $instance['style'], $option_value ); ?>><?php echo esc_html( $option_label ); ?></option>
								<?php } ?>
							</select>
						</li>
						<li>
							<label for="<?php echo $this->get_field_id( 'order' ); ?>"><?php _e( 'Order', $this->textdomain ); ?></label>
							<span class="controlDesc"><?php _e( 'Sort categories with ascending or descending ordering.', $this->textdomain ); ?></span>
							<select id="<?php echo $this->get_field_id( 'order' ); ?>" name="<?php echo $this->get_field_name( 'order' ); ?>">
								<?php foreach ( $order as $option_value => $option_label ) { ?>
									<option value="<?php echo esc_attr( $option_value ); ?>" <?php selected( $instance['order'], $option_value ); ?>><?php echo esc_html( $option_label ); ?></option>
								<?php } ?>
							</select>
						</li>
						<li>
							<label for="<?php echo $this->get_field_id( 'orderby' ); ?>"><?php _e( 'Order By', $this->textdomain ); ?></label> 
							<span class="controlDesc"><?php _e( 'Sort the categories by count, id, slug or term group', $this->textdomain ); ?></span>
							<select id="<?php echo $this->get_field_id( 'orderby' ); ?>" name="<?php echo $this->get_field_name( 'orderby' ); ?>">
								<?php foreach ( $orderby as $option_value => $option_label ) { ?>
									<option value="<?php echo esc_attr( $option_value ); ?>" <?php selected( $instance['orderby'], $option_value ); ?>><?php echo esc_html( $option_label ); ?></option>
								<?php } ?>
							</select>
						</li>
						<li>
							<label for="<?php echo $this->get_field_id( 'depth' ); ?>"><?php _e( 'Depth', $this->textdomain ); ?></label>
							<span class="controlDesc"><?php _e( 'Controls how many levels in the hierarchy of categories are to be included.', $this->textdomain ); ?></span>
							<input type="text" id="<?php echo $this->get_field_id( 'depth' ); ?>" name="<?php echo $this->get_field_name( 'depth' ); ?>" value="<?php echo esc_attr( $instance['depth'] ); ?>" />
						</li>
						<li>
							<label for="<?php echo $this->get_field_id( 'number' ); ?>"><?php _e( 'Number', $this->textdomain ); ?></label>
							<span class="controlDesc"><?php _e( 'Sets the number of categories to display.', $this->textdomain ); ?></span>
							<input type="text" id="<?php echo $this->get_field_id( 'number' ); ?>" name="<?php echo $this->get_field_name( 'number' ); ?>" value="<?php echo esc_attr( $instance['number'] ); ?>" />
						</li>
						<li>
							<label for="<?php echo $this->get_field_id( 'icon_height' ); ?>"><?php _e( 'Thumbnail Height', $this->textdomain ); ?></label>
							<span class="controlDesc"><?php _e( 'The categories image height size in pixels units.', $this->textdomain ); ?></span>
							<input type="text"  id="<?php echo $this->get_field_id( 'icon_height' ); ?>" name="<?php echo $this->get_field_name( 'icon_height' ); ?>" value="<?php echo esc_attr( $instance['icon_height'] ); ?>" />
						</li>
						<li>
							<label for="<?php echo $this->get_field_id( 'icon_width' ); ?>"><?php _e( 'Thumbnail Width:', $this->textdomain ); ?></label>
							<span class="controlDesc"><?php _e( 'The categories image width size in pixels units.', $this->textdomain ); ?></span>
							<input type="text" id="<?php echo $this->get_field_id( 'icon_width' ); ?>" name="<?php echo $this->get_field_name( 'icon_width' ); ?>" value="<?php echo esc_attr( $instance['icon_width'] ); ?>" />
						</li>

						<li>
							<label for="<?php echo $this->get_field_id( 'include' ); ?>"><?php _e( 'Include', $this->textdomain ); ?></label> 
							<select class="widefat" id="<?php echo $this->get_field_id( 'include' ); ?>" name="<?php echo $this->get_field_name( 'include' ); ?>[]" size="4" multiple="multiple">
								<?php foreach ( $terms as $term ) { ?>
									<option value="<?php echo esc_attr( $term->term_id ); ?>" <?php echo ( in_array( $term->term_id, (array) $instance['include'] ) ? 'selected="selected"' : '' ); ?>><?php echo esc_html( $term->name ); ?></option>
								<?php } ?>
							</select>
						</li>
						<li>
							<label for="<?php echo $this->get_field_id( 'exclude' ); ?>"><?php _e( 'Exclude', $this->textdomain ); ?></label> 
							<select class="widefat" id="<?php echo $this->get_field_id( 'exclude' ); ?>" name="<?php echo $this->get_field_name( 'exclude' ); ?>[]" size="4" multiple="multiple">
								<?php foreach ( $terms as $term ) { ?>
									<option value="<?php echo esc_attr( $term->term_id ); ?>" <?php echo ( in_array( $term->term_id, (array) $instance['exclude'] ) ? 'selected="selected"' : '' ); ?>><?php echo esc_html( $term->name ); ?></option>
								<?php } ?>
							</select>
						</li>
					</ul>
				</li>
				
				<li class="tab-pane <?php if ( $instance['toggle_active'][1] ) : ?>active<?php endif; ?>">
					<ul>
						<li>
							<label for="<?php echo $this->get_field_id( 'exclude_tree' ); ?>"><?php _e( 'Eclude Tree', $this->textdomain ); ?></label>
							<input type="text" class="smallfat code" id="<?php echo $this->get_field_id( 'exclude_tree' ); ?>" name="<?php echo $this->get_field_name( 'exclude_tree' ); ?>" value="<?php echo esc_attr( $instance['exclude_tree'] ); ?>" />
							<span class="controlDesc"><?php _e( 'Exclude category-tree from the results.', $this->textdomain ); ?></span>
						</li>
						<li>
							<label for="<?php echo $this->get_field_id( 'child_of' ); ?>"><?php _e( 'Chlid Of', $this->textdomain ); ?></label>
							<input type="text" class="smallfat code" id="<?php echo $this->get_field_id( 'child_of' ); ?>" name="<?php echo $this->get_field_name( 'child_of' ); ?>" value="<?php echo esc_attr( $instance['child_of'] ); ?>" />
							<span class="controlDesc"><?php _e( 'Only display categories that are children of the category identified by this parameter.', $this->textdomain ); ?></span>
						</li>
						<li>
							<label for="<?php echo $this->get_field_id( 'current_category' ); ?>"><?php _e( 'Current Category', $this->textdomain ); ?></label>
							<input type="text" class="smallfat code" id="<?php echo $this->get_field_id( 'current_category' ); ?>" name="<?php echo $this->get_field_name( 'current_category' ); ?>" value="<?php echo esc_attr( $instance['current_category'] ); ?>" />
							<span class="controlDesc"><?php _e( 'Allows you to force the "current-cat" to appear on uses of wp_list_categories that are not on category archive pages', $this->textdomain ); ?></span>
						</li>
						<li>
							<label for="<?php echo $this->get_field_id( 'search' ); ?>"><?php _e( 'Search', $this->textdomain ); ?></label>
							<input type="text" class="widefat code" id="<?php echo $this->get_field_id( 'search' ); ?>" name="<?php echo $this->get_field_name( 'search' ); ?>" value="<?php echo esc_attr( $instance['search'] ); ?>" />
							<span class="controlDesc"><?php _e( 'Only for searching given categories.', $this->textdomain ); ?></span>
						</li>
						<li>
							<label title="Text to show for feed image links on mouse hover" for="<?php echo $this->get_field_id( 'feed' ); ?>"><?php _e( 'Feed', $this->textdomain ); ?></label>
							<input type="text" class="widefat code" id="<?php echo $this->get_field_id( 'feed' ); ?>" name="<?php echo $this->get_field_name( 'feed' ); ?>" value="<?php echo esc_attr( $instance['feed'] ); ?>" />
							<span class="controlDesc"><?php _e( 'Display a link to each category\'s rss-2 feed and set the link text to display.', $this->textdomain ); ?></span>
						</li>
						<li>
							<label for="<?php echo $this->get_field_id( 'feed_type' ); ?>"><?php _e( 'Feed Type', $this->textdomain ); ?></label>
							<span class="controlDesc"><?php _e( 'Show the fed type in atom, rss, rss2, or rdf.', $this->textdomain ); ?></span>
							<select id="<?php echo $this->get_field_id( 'feed_type' ); ?>" name="<?php echo $this->get_field_name( 'feed_type' ); ?>">
								<?php foreach ( $feed_type as $option_value => $option_label ) { ?>
									<option value="<?php echo esc_attr( $option_value ); ?>" <?php selected( $instance['feed_type'], $option_value ); ?>><?php echo esc_html( $option_label ); ?></option>
								<?php } ?>
							</select>
						</li>
						<li>
							<label for="<?php echo $this->get_field_id( 'feed_image' ); ?>"><?php _e( 'Feed Image', $this->textdomain ); ?></label>
							<input type="text" class="widefat code" id="<?php echo $this->get_field_id( 'feed_image' ); ?>" name="<?php echo $this->get_field_name( 'feed_image' ); ?>" value="<?php echo esc_attr( $instance['feed_image'] ); ?>" />
							<span class="controlDesc"><?php _e( 'Set a URI for an image to show as a link.', $this->textdomain ); ?></span>
						</li>
						<li>
							<label for="<?php echo $this->get_field_id( 'hierarchical' ); ?>">
							<input class="checkbox" type="checkbox" <?php checked( $instance['hierarchical'], true ); ?> id="<?php echo $this->get_field_id( 'hierarchical' ); ?>" name="<?php echo $this->get_field_name( 'hierarchical' ); ?>" /> <?php _e( 'Hierarchical', $this->textdomain ); ?></label>
							<span class="controlDesc"><?php _e( 'Set a URI for an image to show as a link.', $this->textdomain ); ?></span>
						</li>
						<li>
							<label for="<?php echo $this->get_field_id( 'use_desc_for_title' ); ?>">
							<input class="checkbox" type="checkbox" <?php checked( $instance['use_desc_for_title'], true ); ?> id="<?php echo $this->get_field_id( 'use_desc_for_title' ); ?>" name="<?php echo $this->get_field_name( 'use_desc_for_title' ); ?>" /> <?php _e( 'Use description', $this->textdomain ); ?></label>
							<span class="controlDesc"><?php _e( 'Sets whether a category\'s description is inserted into the title attribute of the links created.', $this->textdomain ); ?></span>
						</li>			
						<li>
							<label for="<?php echo $this->get_field_id( 'show_last_update' ); ?>">
							<input class="checkbox" type="checkbox" <?php checked( $instance['show_last_update'], true ); ?> id="<?php echo $this->get_field_id( 'show_last_update' ); ?>" name="<?php echo $this->get_field_name( 'show_last_update' ); ?>" /> <?php _e( 'Show last update', $this->textdomain ); ?></label>
							<span class="controlDesc"><?php _e( 'Show categories with last updated', $this->textdomain ); ?></span>
						</li>
						<li>
							<label for="<?php echo $this->get_field_id( 'show_count' ); ?>">
							<input class="checkbox" type="checkbox" <?php checked( $instance['show_count'], true ); ?> id="<?php echo $this->get_field_id( 'show_count' ); ?>" name="<?php echo $this->get_field_name( 'show_count' ); ?>" /> <?php _e( 'Show count', $this->textdomain ); ?></label>
							<span class="controlDesc"><?php _e( 'Toggles the display of the current count of posts in each category.', $this->textdomain ); ?></span>
						</li>
						<li>
							<label for="<?php echo $this->get_field_id( 'hide_empty' ); ?>">
							<input class="checkbox" type="checkbox" <?php checked( $instance['hide_empty'], true ); ?> id="<?php echo $this->get_field_id( 'hide_empty' ); ?>" name="<?php echo $this->get_field_name( 'hide_empty' ); ?>" /> <?php _e( 'Hide empty', $this->textdomain ); ?></label>
							<span class="controlDesc"><?php _e( 'Toggles the display of categories with no posts.', $this->textdomain ); ?></span>
						</li>
					</ul>
				</li>
				
				<li class="tab-pane <?php if ( $instance['toggle_active'][2] ) : ?>active<?php endif; ?>">
					<ul>
						<li>
							<label for="<?php echo $this->get_field_id( 'title_icon' ); ?>"><?php _e( 'Widget Title Icon', $this->textdomain ); ?></label>
							<span class="controlDesc"><?php _e( 'Attach an image or icon for the widget title', $this->textdomain ); ?></span>
							<img alt="" class="optionImage" src="<?php echo esc_attr( $instance['title_icon'] ); ?>">
							<a href="#" class="addImage button"><?php _e( 'Add Image', $this->textdomain ); ?></a>
							<a class="<?php if ( empty($instance['title_icon'] ) ) : ?>hideRemove<?php else : ?>showRemove<?php endif; ?> removeImage button" href="#"><?php _e('Remove', 'dropline'); ?></a>
							<input type="hidden" id="<?php echo $this->get_field_id( 'title_icon' ); ?>" name="<?php echo $this->get_field_name( 'title_icon' ); ?>" value="<?php echo esc_attr( $instance['title_icon'] ); ?>" />
						</li>
						<li>
							<label for="<?php echo $this->get_field_id( 'bgImage' ); ?>"><?php _e( 'Background Image', $this->textdomain ); ?></label>
							<span class="controlDesc"><?php _e( 'The counter background image.', $this->textdomain ); ?></span>
							<img alt="" class="optionImage" src="<?php echo esc_attr( $instance['bgImage'] ); ?>">
							<a href="#" class="addImage button"><?php _e( 'Add Image', $this->textdomain ); ?></a>
							<a class="<?php if ( empty($instance['bgImage'] ) ) : ?>hideRemove<?php else : ?>showRemove<?php endif; ?> removeImage button" href="#"><?php _e('Remove', 'dropline'); ?></a>
							<input type="hidden" id="<?php echo $this->get_field_id( 'bgImage' ); ?>" name="<?php echo $this->get_field_name( 'bgImage' ); ?>" value="<?php echo esc_attr( $instance['bgImage'] ); ?>" />
						</li>
						<li>
							<label for="<?php echo $this->get_field_id('intro_text'); ?>"><?php _e( 'Intro text:', $this->textdomain ); ?></label>
							<span class="controlDesc"><?php _e( 'This option will display addtional text before the widget content and supports HTML.', $this->textdomain ); ?></span>
							<textarea name="<?php echo $this->get_field_name( 'intro_text' ); ?>" id="<?php echo $this->get_field_id( 'intro_text' ); ?>" rows="3" class="widefat"><?php echo $intro_text; ?></textarea>
						</li>
						<li>
							<label for="<?php echo $this->get_field_id('outro_text'); ?>"><?php _e( 'Outro text:', $this->textdomain ); ?></label>
							<span class="controlDesc"><?php _e( 'This option will display addtional text after widget and supports HTML.', $this->textdomain ); ?></span>
							<textarea name="<?php echo $this->get_field_name( 'outro_text' ); ?>" id="<?php echo $this->get_field_id( 'outro_text' ); ?>" rows="3" class="widefat"><?php echo $outro_text; ?></textarea>
						</li>	
						<li>
							<label for="<?php echo $this->get_field_id('customstylescript'); ?>"><?php _e( 'Custom Script & Stylesheet :', $this->textdomain );?></label>
							<span class="controlDesc"><?php _e( 'Use this box for additional widget CSS style of custom javascript. Current widget selector: ', $this->textdomain ); ?><?php echo '<code>#' . $this->id . '</code>'; ?></span>
							<textarea name="<?php echo $this->get_field_name( 'customstylescript' ); ?>" id="<?php echo $this->get_field_id( 'customstylescript' ); ?>" rows="4" class="widefat code"><?php echo htmlentities($instance['customstylescript']); ?></textarea>
						</li>
					</ul>
				</li>
				
				<li class="tab-pane <?php if ( $instance['toggle_active'][3] ) : ?>active<?php endif; ?>">
					<ul>
						<li>
							<h3>Support and Contribute</h3>
							<p>Please ask us for supports or new features here:<p>
							<a href="http://codecanyon.net/user/zourbuth#message?ref=zourbuth">CodeCanyon Author Profile Page</a><br />
							<a href="http://codecanyon.net/item/advanced-categories-widget/discussion/422878?ref=zourbuth">Plugin Discussion</a>
						</li>
					</ul>
				</li>
			</ul>
		</div>
	<?php
	}
}

?>