<?php

/**
 * Comments widget class
 *
 * @since 2.8.0
 */
class Advanced_Comments_Widget extends WP_Widget {

	function __construct() {
	
		/* Set up the widget control options. */
		$control_options = array(
			'width' => 800,
			'height' => 350
		);
		
		$widget_ops = array('classname' => 'advanced_comments', 'description' => __( '[+] Show comments in advanced usage' ) );
		parent::__construct('advanced_comments', __('Advanced Comments'), $widget_ops, $control_options);
		$this->alt_option_name = 'advanced_comments';

		/* load the widget stylesheet for the widgets admin screen. */
		add_action( 'load-widgets.php', array(&$this, 'advanced_comments_admin_style') );
		
		if ( is_active_widget(false, false, $this->id_base) ) {
			add_action( 'wp_head', array(&$this, 'comments_style') );
			wp_enqueue_style("advanced-comments", ADVANCED_COMMENTS_WIDGET_URL . "css/advanced-comments.css");
			add_action( 'wp_head', array( &$this, 'print_custom_styles_cript') ); // print the user costum style sheet.
		}

		add_action( 'comment_post', array(&$this, 'flush_widget_cache') );
		add_action( 'transition_comment_status', array(&$this, 'flush_widget_cache') );
	}
	
	/* Push the widget stylesheet widget.css into widget admin page */
	function advanced_comments_admin_style() {
		wp_enqueue_style( 'advanced-categories-widget-admin', ADVANCED_COMMENTS_WIDGET_DIR . 'css/widget-admin.css', false, 0.7, 'screen' );
	}
	
	function comments_style() {
		if ( ! current_theme_supports( 'widgets' ) // Temp hack #14876
			|| ! apply_filters( 'show_comments_widget_style', true, $this->id_base ) )
			return;
		?>
	<style type="text/css">.widget_comments a{display:inline !important;padding:0 !important;margin:0 !important;}</style>
<?php
	}
	
	/* Print the custom script-style to the header wp_head */
	function print_custom_styles_cript() {
		$all_widgets = $this->get_settings();
		foreach ($all_widgets as $key => $rc_setting){
			$widget_id = $this->id_base . '-' . $key;
			if( is_active_widget( false, $widget_id, $this->id_base ) ){
				if ( !empty( $rc_setting['customstylescript'] ) )
					echo $rc_setting['customstylescript'];
			}
		}		
	}
	
	function flush_widget_cache() {
		wp_cache_delete('widget_comments', 'widget');
	}

	function widget( $args, $instance ) {
		global $comments, $comment;

		$cache = wp_cache_get('widget_comments', 'widget');

		if ( ! is_array( $cache ) )
			$cache = array();

		if ( isset( $cache[$args['widget_id']] ) ) {
			echo $cache[$args['widget_id']];
			return;
		}

 		extract($args, EXTR_SKIP);
 		$output = '';
 		$title 				= apply_filters('widget_title', empty($instance['title']) ? __('Comments') : $instance['title']);
 		$avatar_size 		= apply_filters('avatar_size', empty($instance['avatar_size']) ? __('Avatar Size') : $instance['avatar_size']);
 		$status 			= apply_filters('status', empty($instance['status']) ? __('Status') : $instance['status']);
 		$orderby 			= apply_filters('orderby', empty($instance['orderby']) ? __('Order by') : $instance['orderby']);
 		$order 				= apply_filters('order', empty($instance['order']) ? __('Order') : $instance['order']);
 		$empty_avatar_url	= apply_filters('empty_avatar_url', empty($instance['empty_avatar_url']) ? __('') : $instance['empty_avatar_url']);
 		$date_format 		= apply_filters('date_format', empty($instance['date_format']) ? __('jS, F Y h:i:s A') : $instance['date_format']);
 		$post_id 			= $instance['post_id'];
 		$post_author 		= !empty( $instance['post_author'] ) ? join( ', ', $instance['post_author'] ) : '';
 		$post_type 			= $instance['post_type'];
 		$search 			= $instance['search'];
 		$type 				= $instance['type'];
 		$comment_count 		= $instance['comment_count'] ? '1' : '0';
 		$comment_date 		= $instance['comment_date'] ? '1' : '0';
 		$comment_content 	= $instance['comment_content'] ? '1' : '0';
 		$show_avatar 		= $instance['show_avatar'] ? '1' : '0';
 		$show_date 			= $instance['show_date'] ? '1' : '0';
		
		$intro_text = apply_filters( 'intro_text', $instance['intro_text'] );
		$outro_text = apply_filters( 'outro_text', $instance['outro_text'] );
		$customstylescript = apply_filters( 'customstylescript', $instance['customstylescript'] );


		if ( ! $number = absint( $instance['number'] ) )
 			$number = 5;

		$comments = get_comments( array( 
			'number'	 	=> $number, 
			'status' 		=> $status,
			'post_status' 	=> 'publish',
			'orderby'		=> $orderby,
			'order'			=> $order,
			'post_id'		=> $post_id,
			'post_author'	=> $post_author,
			'post_type'		=> $post_type,
			'type'			=> $type,
			'search'		=> $search
		) );
		//print_r ($post_type);
		$output .= $before_widget;
		if ( $title )
			$output .= $before_title . $title . $after_title;
		
		/* Print intro text if exist */
		if ( !empty( $instance['intro_text'] ) )
			$output .= '<p class="'. $this->id . '-intro-text">' . $instance['intro_text'] . '</p>';
			
		$output .= '<ul id="widget_comments">';
		if ( $comments ) {
			foreach ( (array) $comments as $comment) {
				$output .= '<li class="widget_comments">';

				if ( $show_avatar )
					$output .= get_avatar( $comment->comment_author_email, $avatar_size , $empty_avatar_url );
			
				$output .=  /* translators: comments widget: 1: comment author, 2: post link */ sprintf(_x('%1$s <span class="meta_on">on</span> %2$s', 'widgets'), get_comment_author_link(), '<a href="' . esc_url( get_comment_link($comment->comment_ID) ) . '">' . get_the_title($comment->comment_post_ID) . '</a>');
				
				if ( $comment_count )
					$output .= ' ( <a class="comment_count" href="' . get_comments_link(  $comment->comment_post_id ) . '">' . $comment->comment_count . '</a> )';

				if ( $show_date )
					$output .= '<span class="comment_date">' . date_format( date_create($comment->comment_date), $date_format ) . '</span>';
				
				if ( $comment_content )
					$output .= '<span class="comment_content">' . $comment->comment_content . '</span>';
					
				$output .= '</li>';
			}
 		}
		$output .= '</ul>';
		
		/* Print outro text if exist */
		if ( !empty( $instance['outro_text'] ) )
			$output .= '<p class="'. $this->id . '-outro_text">' . $instance['outro_text'] . '</p>';
			
		$output .= $after_widget;

		echo $output;

		$cache[$args['widget_id']] = $output;
		wp_cache_set('widget_comments', $cache, 'widget');
	}

	function update( $new_instance, $old_instance ) {
		$instance = $old_instance;
		$instance['title']				= strip_tags($new_instance['title']);
		$instance['number']				= absint( $new_instance['number'] );
		$instance['avatar_size']		= absint( $new_instance['avatar_size'] );
		$instance['status']				= strip_tags( $new_instance['status'] );
		$instance['orderby']			= strip_tags( $new_instance['orderby'] );
		$instance['order']				= strip_tags( $new_instance['order'] );
		$instance['empty_avatar_url']	= esc_url( $new_instance['empty_avatar_url'] );
		$instance['date_format']		= strip_tags( $new_instance['date_format'] );
		$instance['post_id']			= strip_tags( $new_instance['post_id'] );
		$instance['post_author']		= (array) $new_instance['post_author'];
		$instance['post_type']			= strip_tags($new_instance['post_type']);
		$instance['type']				= strip_tags($new_instance['type']);
		$instance['search']				= strip_tags($new_instance['search']);
		$instance['comment_count']		= !empty($new_instance['comment_count']) ? 1 : 0;
		$instance['comment_content']	= !empty($new_instance['comment_content']) ? 1 : 0;
		$instance['show_avatar']		= !empty($new_instance['show_avatar']) ? 1 : 0;
		$instance['show_date']			= !empty($new_instance['show_date']) ? 1 : 0;
		$instance['intro_text']			= $new_instance['intro_text'];
		$instance['outro_text']			= $new_instance['outro_text'];
		$instance['customstylescript']	= $new_instance['customstylescript'];
		
		$this->flush_widget_cache();

		$alloptions = wp_cache_get( 'alloptions', 'options' );
		if ( isset($alloptions['widget_comments']) )
			delete_option('widget_comments');

		return $instance;
	}

	function form( $instance ) {
		$title				= isset($instance['title']) ? esc_attr($instance['title']) : '';
		$number				= isset($instance['number']) ? absint($instance['number']) : 5;
		$avatar_size		= isset($instance['avatar_size']) ? absint($instance['avatar_size']) : 40;
		$status				= isset($instance['status']) ? esc_attr($instance['status']) : 'approve';
		$status_item		= array( 
									'hold' => esc_attr__( 'hold', $this->textdomain ), 
									'approve' => esc_attr__( 'approve', $this->textdomain ), 
									'spam' => esc_attr__( 'spam', $this->textdomain ),
									'trash' => esc_attr__( 'trash', $this->textdomain )
							);
		$orderby			= isset($instance['orderby']) ? esc_attr($instance['orderby']) : 'comment_date_gmt';
		$orderby_item		= array( 
									'' => esc_attr__( '', $this->textdomain ),
									'comment_agent' => esc_attr__( 'comment_agent', $this->textdomain ), 
									'comment_approved' => esc_attr__( 'comment_approved', $this->textdomain ), 
									'comment_author' => esc_attr__( 'comment_author', $this->textdomain ),
									'comment_author_email' => esc_attr__( 'comment_author_email', $this->textdomain ),
									'comment_author_IP' => esc_attr__( 'comment_author_IP', $this->textdomain ),
									'comment_author_url' => esc_attr__( 'comment_author_url', $this->textdomain ),
									'comment_content' => esc_attr__( 'comment_content', $this->textdomain ),
									'comment_date' => esc_attr__( 'comment_date', $this->textdomain ),
									'comment_date_gmt' => esc_attr__( 'comment_date_gmt', $this->textdomain ),
									'comment_ID' => esc_attr__( 'comment_ID', $this->textdomain ),
									'comment_karma' => esc_attr__( 'comment_karma', $this->textdomain ),
									'comment_parent' => esc_attr__( 'comment_parent', $this->textdomain ),
									'comment_post_id' => esc_attr__( 'comment_post_id', $this->textdomain ),
									'comment_type' => esc_attr__( 'comment_type', $this->textdomain ),
									'user_id' => esc_attr__( 'user_id', $this->textdomain )
							);
		$order				= isset( $instance['order'] ) ? esc_attr($instance['order']) : 'DESC';
		$order_item			= array( 'ASC' => esc_attr__( 'Ascending', $this->textdomain ), 'DESC' => esc_attr__( 'Descending', $this->textdomain ) );
		$empty_avatar_url	= $instance['empty_avatar_url'];
		$date_format		= isset( $instance['date_format']) ? esc_attr($instance['date_format']) : 'jS, F Y h:i:s A';
		$post_id			= $instance['post_id'];
		
		$post_author		= array();
		$posts = get_posts( array( 'post_status' => 'any', 'post_mime_type' => '', 'orderby' => 'title', 'order' => 'ASC', 'numberposts' => -1 ) );
		$authors = array();
		foreach ( $posts as $post )
			$authors[$post->post_author] = get_the_author_meta( 'display_name', $post->post_author );

		$post_type			= array();
		$post_types 		= get_post_types();

		$search				= $instance['search'];
		
		$type				= $instance['type'];
		$types				= array( 'comment' => esc_attr__( 'comment', $this->textdomain ), 'pings' => esc_attr__( 'pings', $this->textdomain ) );
		
		$comment_count		= isset( $instance['comment_count'] ) ? (bool) $instance['comment_count'] : false;
		$comment_content	= isset( $instance['comment_content'] ) ? (bool) $instance['comment_content'] : false;
		$show_avatar		= isset( $instance['show_avatar'] ) ? (bool) $instance['show_avatar'] : true;
		$show_date			= isset( $instance['show_date'] ) ? (bool) $instance['show_date'] : true;
		$intro_text			= $instance['intro_text'];
		$outro_text			= $instance['outro_text'];
		$customstylescript	= $instance['customstylescript'];
?>
		<div class="zframe-widget-controls columns-3">
			<p><label for="<?php echo $this->get_field_id('title'); ?>"><?php _e('Title:'); ?></label>
			<input class="widefat" id="<?php echo $this->get_field_id('title'); ?>" name="<?php echo $this->get_field_name('title'); ?>" type="text" value="<?php echo $title; ?>" /></p>

			<p><label for="<?php echo $this->get_field_id('number'); ?>"><?php _e('Number of comments to show:'); ?></label>
			<input class="smallfat" id="<?php echo $this->get_field_id('number'); ?>" name="<?php echo $this->get_field_name('number'); ?>" type="text" value="<?php echo $number; ?>" size="3" /></p>
				
			<p><label for="<?php echo $this->get_field_id( 'date_format' ); ?>"><code>date_format</code></label>
			<input type="text" style="width: 130px" class="smallfat code" id="<?php echo $this->get_field_id( 'date_format' ); ?>" name="<?php echo $this->get_field_name( 'date_format' ); ?>" value="<?php echo $date_format; ?>" /></p>
						
			<p>
				<label for="<?php echo $this->get_field_id( 'status' ); ?>"><code>status</code></label> 
				<select class="widefat" id="<?php echo $this->get_field_id( 'status' ); ?>" name="<?php echo $this->get_field_name( 'status' ); ?>">
					<?php foreach ( $status_item as $option_value => $option_label ) { ?>
						<option value="<?php echo $option_value; ?>" <?php selected( $status, $option_value ); ?>><?php echo $option_label; ?></option>
					<?php } ?>
				</select>
			</p>

			<p>
				<label for="<?php echo $this->get_field_id( 'order' ); ?>"><code>order</code></label> 
				<select class="widefat" id="<?php echo $this->get_field_id( 'order' ); ?>" name="<?php echo $this->get_field_name( 'order' ); ?>">
					<?php foreach ( $order_item as $option_value => $option_label ) { ?>
						<option value="<?php echo $option_value; ?>" <?php selected( $order, $option_value ); ?>><?php echo $option_label; ?></option>
					<?php } ?>
				</select>
			</p>

			<p >
				<label for="<?php echo $this->get_field_id( 'orderby' ); ?>"><code>orderby</code></label> 
				<select class="widefat" id="<?php echo $this->get_field_id( 'orderby' ); ?>" name="<?php echo $this->get_field_name( 'orderby' ); ?>">
					<?php foreach ( $orderby_item as $option_value => $option_label ) { ?>
						<option value="<?php echo $option_value; ?>" <?php selected( $orderby, $option_value ); ?>><?php echo $option_label; ?></option>
					<?php } ?>
				</select>
			</p>
			
			<p><label for="<?php echo $this->get_field_id('avatar_size'); ?>"><?php _e('Avatar size:'); ?></label>
			<input class="smallfat" id="<?php echo $this->get_field_id('avatar_size'); ?>" name="<?php echo $this->get_field_name('avatar_size'); ?>" type="text" value="<?php echo $avatar_size; ?>" size="3" /></p>

			<p><label for="<?php echo $this->get_field_id('empty_avatar_url'); ?>"><?php _e('Empty avatar URL:'); ?></label>
			<input class="widefat code" id="<?php echo $this->get_field_id('empty_avatar_url'); ?>" name="<?php echo $this->get_field_name('empty_avatar_url'); ?>" type="text" value="<?php echo $empty_avatar_url; ?>" size="3" /></p>
			
			<p><input type="checkbox" class="checkbox" id="<?php echo $this->get_field_id('show_avatar'); ?>" name="<?php echo $this->get_field_name('show_avatar'); ?>"<?php checked( $show_avatar ); ?> />
			<label for="<?php echo $this->get_field_id('show_avatar'); ?>"><code>show_avatar</code></label></p>
			
			<p><input type="checkbox" class="checkbox" id="<?php echo $this->get_field_id('show_date'); ?>" name="<?php echo $this->get_field_name('show_date'); ?>"<?php checked( $show_date ); ?> />
			<label for="<?php echo $this->get_field_id('show_date'); ?>"><code>show_date</code></label></p>
		</div>
		
		<div class="zframe-widget-controls columns-3">
			<p>
				<label for="<?php echo $this->get_field_id( 'post_author' ); ?>"><code>post_author</code></label> 
				<select class="widefat" id="<?php echo $this->get_field_id( 'post_author' ); ?>" name="<?php echo $this->get_field_name( 'post_author' ); ?>[]" size="4" multiple="multiple">
					<?php foreach ( $authors as $option_value => $option_label ) { ?>
						<option value="<?php echo esc_attr( $option_value ); ?>" <?php echo ( in_array( $option_value, (array) $instance['post_author'] ) ? 'selected="selected"' : '' ); ?>><?php echo esc_html( $option_label ); ?></option>
					<?php } ?>
				</select>
			</p>
			<p>
				<label for="<?php echo $this->get_field_id( 'post_type' ); ?>"><code>post_type</code></label> 
				<select class="widefat" id="<?php echo $this->get_field_id( 'post_type' ); ?>" name="<?php echo $this->get_field_name( 'post_type' ); ?>">
					<?php foreach ( $post_types as $option_value => $option_label ) { ?>
						<option value="<?php echo $option_value; ?>" <?php selected( $instance['post_type'], $option_value ); ?>><?php echo $option_label; ?></option>
					<?php } ?>
				</select>
			</p>
			
			<p><label for="<?php echo $this->get_field_id( 'post_id' ); ?>"><code>post_id</code></label>
			<input type="text" class="smallfat" id="<?php echo $this->get_field_id( 'post_id' ); ?>" name="<?php echo $this->get_field_name( 'post_id' ); ?>" value="<?php echo $post_id; ?>" /></p>

			<p>
				<label for="<?php echo $this->get_field_id( 'type' ); ?>"><code>type</code></label> 
				<select class="widefat" id="<?php echo $this->get_field_id( 'type' ); ?>" name="<?php echo $this->get_field_name( 'type' ); ?>">
					<?php foreach ( $types as $option_value => $option_label ) { ?>
						<option value="<?php echo $option_value; ?>" <?php selected( $instance['type'], $option_value ); ?>><?php echo $option_label; ?></option>
					<?php } ?>
				</select>
			</p>
			
			<p><label for="<?php echo $this->get_field_id('search'); ?>"><code>search</code></label>
			<input class="widefat code" id="<?php echo $this->get_field_id('search'); ?>" name="<?php echo $this->get_field_name('search'); ?>" type="text" value="<?php echo $search; ?>" size="3" /></p>

			<p><input type="checkbox" class="checkbox" id="<?php echo $this->get_field_id('comment_content'); ?>" name="<?php echo $this->get_field_name('comment_content'); ?>"<?php checked( $comment_content ); ?> />
			<label for="<?php echo $this->get_field_id('comment_content'); ?>"><code>comment_content</code></label></p>
			
			<p><input type="checkbox" class="checkbox" id="<?php echo $this->get_field_id('comment_count'); ?>" name="<?php echo $this->get_field_name('comment_count'); ?>"<?php checked( $comment_count ); ?> />
			<label for="<?php echo $this->get_field_id('comment_count'); ?>"><code>comment_count</code></label></p>	
		</div>
		
		<div class="zframe-widget-controls columns-3 column-last">			
			<p><label for="<?php echo $this->get_field_id( 'intro_text' ); ?>"><?php _e('Intro Text:', $this->textdomain ) ?></label><br />
			<textarea name="<?php echo $this->get_field_name( 'intro_text' ); ?>" id="<?php echo $this->get_field_id( 'intro_text' ); ?>" rows="4" class="widefat"><?php echo htmlentities($instance['intro_text']); ?></textarea></p>
			
			<p><label for="<?php echo $this->get_field_id( 'outro_text' ); ?>"><?php _e('Outro Text:', $this->textdomain ) ?></label><br />
			<textarea name="<?php echo $this->get_field_name( 'outro_text' ); ?>" id="<?php echo $this->get_field_id( 'outro_text' ); ?>" rows="4" class="widefat"><?php echo htmlentities($instance['outro_text']); ?></textarea></p>
			
			<p><label for="<?php echo $this->get_field_id('customstylescript'); ?>"><?php _e( 'Custom Script & Stylesheet :', $this->textdomain );  echo '<br />Widget ID: <code>#' . $this->id . '</code>' ; ?>
			<textarea style="font-size: 11px;" name="<?php echo $this->get_field_name( 'customstylescript' ); ?>" id="<?php echo $this->get_field_id( 'customstylescript' ); ?>" rows="5" class="widefat code"><?php echo htmlentities($instance['customstylescript']); ?></textarea></label></p>
		</div>
		
		<div style="clear:both;">&nbsp;</div>
<?php
	}
}

?>