<?php
/**
 * Default Geo Mashup Search results template.
 *
 * THIS FILE WILL BE OVERWRITTEN BY AUTOMATIC UPGRADES
 * See the geo-mashup-search.php plugin file for license
 *
 * Copy this to a file named geo-mashup-search-results.php in your active theme folder
 * to customize. For bonus points delete this message in the copy!
 *
 * Variables in scope:
 * $geo_mashup_search  object   The managing search object
 * $result_count       int      The number of objects found
 * $search_text        string   The search text entered in the form
 * $radius             int      The search radius
 * $units              string   'mi' or 'km'
 * $object_name        string   'post' or 'user' or 'comment'
 * $near_location      array    The location searched, including 'lat' and 'lng'
 * $distance_factor    float    The multiplier to convert the radius to kilometers
 * $approximate_zoom   int      A guess at a zoom level that will include all results
 *
 * Methods of $geo_mashup_search mimic WordPress Loop functions have_posts()
 * and the_post() (see http://codex.wordpress.org/The_Loop). This makes post
 * template functions like the_title() work as expected. For distance:
 *
 * $geo_mashup_search->the_distance();
 *
 * will echo the distance with units. Its output can be modified:
 *
 * $geo_mashup_search->the_distance( 'decimal_places=1&append_units=0&echo=0' );
 */
?>
<div id="geo-mashup-search-results">

	<h2><?php _e( 'Search results near', 'GeoMashup' ); ?> "<?php echo $search_text; ?>"</h2>
	
	<?php if ( $geo_mashup_search->have_posts() ) : ?>

		<?php if ($object_name == 'post'):?>

			<?php while ( $geo_mashup_search->have_posts() ) : $geo_mashup_search->the_post(); ?>
					<div class="search-result">
						<h3><a href="<?php the_permalink(); ?>" title=""><?php the_title(); ?></a></h3>
						<p><?php the_excerpt(); ?></p>
						<p>
					<?php _e( 'Distance', 'GeoMashup' ); ?>:
					<?php $geo_mashup_search->the_distance(); ?>
				</p>
			</div>
			<?php endwhile; ?>
		<?php elseif ($object_name == 'user'):?>
				
			<?php while ( $geo_mashup_search->have_posts() ) : $user=$geo_mashup_search->get_userdata(); ?>

			<?php
            
            $userID = $user->id;
            $queryID = 'user_'.$userID;
            $instructor_email = get_the_author_meta('user_email', $userID);
            $user_bio = substr(get_field('bio', $queryID),0,800);
            $address = get_sub_field('address');
            $photo = get_field('profile_image', $queryID);
            $button = '<button onclick="location.href=\''.THEME_URI.'/profile/?instructor='.$user->ID.'\'"><span class="cufonButtonReg">More Info.</span></button></div>';
            
            if ( !empty($photo) ) {
            
                if ( !empty($photo['sizes']['medium']) ) {
                    
                    $photo = THEME_URI.'/inc/thumb.php?src='.$photo['sizes']['medium'].'&w=180&h=190&q=95';
                    
                } else {
                    
                    $photo = THEME_URI.'/inc/thumb.php?src='.$photo['sizes']['full'].'&w=180&h=190&q=95';
                    
                }
            
            } else {
                
                $photo = THEME_URI.'/images/no_image.png';
                
            }
    
            
            ?>
			<div class="search-result">
				<table>
                       <tbody>
                       	<tr>
						<td><div class="geo-result" id="p-img">
						<img class="profile" width="200" height="200" src="<?php echo $photo; ?>" />
						</br><h4><?php echo $user->first_name.' '.$user->last_name;?></h4>
						</br><p>teaches <?php $geo_mashup_search->the_distance( 'append_units=0' ); ?> miles away</p></div></td>
						<td id="schedule"><div class="geo-result" id="schedule">
								  <?php 
								  		            
            $display = '';
            
            if ( get_field('announcements', $queryID) ) :
                
                $announcement = get_field('announcements', $queryID);
                
                $display .= '<div class="grid_12 alpha omega mb10" style="display:none;">
                                <p class="instructorNote">'.$announcement.'</p>
                             </div>';
                             
            endif;
            
            $display .= '<div class="grid_12 alpha omega clearfix">';
			
			$display .= '<h2 class="cufon instructor sub">Classes</h2>';
            
            if( get_field('classes', $queryID) ) :
                
                $display .= '<table id="regular_classes" class="classList userList" width="100%" border="0" cellpadding="0" cellspacing="0">
                             <tbody>';				
                
                while( has_sub_field('classes', $queryID) ) : 
                
                    $frequency = get_sub_field('frequency');
                    $day = get_sub_field('day');
                    $time = get_sub_field('time');
                    $gym = get_sub_field('gym_name');
                    $address = get_sub_field('address');
                    $state = get_sub_field('state');
					$state_clean = convert_state( get_sub_field('state') );
                    $city = get_sub_field('city');
					$postal = get_sub_field('zipcode');
                    $today = strtotime(date('Y-m-d'));
                    $classdate = strtotime($date);						
                    
                    $display .= '<tr>
                                    <td><b>'.$frequency.' '.$day.'</b> '.$time.'</td>
                                    <td>'.$gym.'</br><i>'.$address.'</i></td>
                                 </tr>';
                                    
                endwhile;
                
            $display .= '</tbody></table>';
            $isOther = 'Other ';
            
            else: 
			
				$display .= '<p><b style="cursor:pointer; color:#4b92db;" onclick="location.href=\''.THEME_URI.'/profile/?instructor='.$user->ID.'\'">Contact</b> this Instructor for a list of classes.</p>';
			
			endif;
            
            if( get_field('other_classes', $queryID) ) :
                
                $display .= '<table id="other_classes" class="classList userList" width="100%" border="0" cellpadding="0" cellspacing="0">
                             <tbody>';
                             
                $count = 0;				
                
                while( has_sub_field('other_classes', $queryID) ) :
                
                    $date = get_sub_field('date');
                    $date_object = new DateTime($date);
                    $class_date = $date_object->format('l F jS');			
                    $time = get_sub_field('time');
                    $gym = get_sub_field('gym_name');
                    $state = get_sub_field('state');
					$state_clean = convert_state( get_sub_field('state') );
                    $city = get_sub_field('city');
					$postal = get_sub_field('zipcode');
                    $address = get_sub_field('address');
                    $today = strtotime(date('Y-m-d'));
                    $classdate = strtotime($date);	
                    
                    if ( $classdate > $today ) :
                    
                        $display .= '<tr>
                                        <td><b>'.$class_date.'</b> '.$time.'</td>
	                                    <td>'.$gym.'</td>
                                     </tr>';
                                 
                    endif;
                                    
                $count++;
                
                endwhile;
                
            $display .= '</tbody></table>';
            
            endif;
            
            if ( $count == 0 ) :
                
                $display .= "<script>
                                $(document).ready(function() {
                                    
                                    $('#other_classes').remove();
                                    
                                });
                             </script>";
            
            endif;
            
            $display .= '</div>';
            
            echo $display;					
            
            ?>

						        <?php echo $button; ?></div></div>	</td>
									</tr>
								</tbody>
							   </table>
					</div>
			<?php endwhile; ?>		
		<?php endif;?>
	<?php else : ?>

				<p id="none-found">Unfortunately, it doesn't look like there are any WERQ classes near your search area. <a href="mailto:Info@WERQFitness.com">Contact us</a> to learn how to bring a WERQ Fitness class to a location near you!</p>

	<?php endif; ?>
</div>